@@ -8,15 +8,16 @@ use Carp;
 
 =head1 NAME
 
-Queue::DBI::Element - An object representing an element pulled from the queue
+Queue::DBI::Element - An object representing an element pulled from the queue.
+
 
 =head1 VERSION
 
-Version 1.7.3
+Version 1.8.0
 
 =cut
 
-our $VERSION = '1.7.3';
+our $VERSION = '1.8.0';
 
 
 =head1 SYNOPSIS
@@ -402,14 +403,12 @@ L<http://search.cpan.org/dist/Queue-DBI/>
 =head1 ACKNOWLEDGEMENTS
 
 Thanks to ThinkGeek (L<http://www.thinkgeek.com/>) and its corporate overlords
-at Geeknet (L<http://www.geek.net/>), for footing the bill while I eat pizza
-and write code for them!
+at Geeknet (L<http://www.geek.net/>), for footing the bill while I write code
+for them!
 
 Thanks to Jacob Rose C<< <jacob at thinkgeek.com> >>, who wrote the first
 queueing module at ThinkGeek L<http://www.thinkgeek.com> and whose work
-provided the inspiration to write this full-fledged queueing system. His
-contribution to shaping the original API in version 1.0.0 was also very
-valuable.
+provided the inspiration to write this full-fledged queueing system.
 
 Thanks to Jamie McCarthy for the locking mechanism improvements in version 1.1.0.
 
